@@ -1,10 +1,11 @@
 # standard library
 from __future__ import annotations
+
 import asyncio
 import binascii
-import os
 import ipaddress
 import logging
+import os
 
 # 3rd party
 import aiofiles
@@ -13,9 +14,9 @@ from aiotinyrpc.protocols.jsonrpc import JSONRPCProtocol
 from aiotinyrpc.server import RPCServer
 from aiotinyrpc.transports.socket import EncryptedSocketServerTransport
 
+from fluxvault.extensions import FluxVaultExtensions
 # this package
 from fluxvault.helpers import get_app_and_component_name
-from fluxvault.extensions import FluxVaultExtensions
 
 
 class FluxAgentException(Exception):
@@ -48,6 +49,7 @@ class FluxAgent:
         working_dir: str = "/tmp",
         whitelisted_addresses: list = ["127.0.0.1"],
         authenticate_vault: bool = True,
+        manage_loop: bool = True,
     ):
         self.app = web.Application()
         self.enable_local_fileserver = enable_local_fileserver
@@ -55,6 +57,7 @@ class FluxAgent:
         self.log = self.get_logger()
         self.local_fileserver_port = local_fileserver_port
         self.loop = asyncio.get_event_loop()
+        self.manage_loop = manage_loop
         self.managed_files = managed_files
         self.ready_to_serve = False
         self.runners = []
@@ -100,14 +103,15 @@ class FluxAgent:
             self.log.info(
                 f"Local file server running on port {self.local_fileserver_port}"
             )
-        #  ToDo: this needs to be modified so it gets added to loop, then loop forever
-        self.rpc_server.serve_forever()
 
-        # try:
-        #     loop.run_forever()
-        # finally:
-        #     for runner in self.runners:
-        #         loop.run_until_complete(runner.cleanup())
+        self.loop.create_task(self.rpc_server.serve_forever())
+
+        if self.manage_loop:
+            try:
+                self.loop.run_forever()
+            finally:
+                for runner in self.runners:
+                    self.loop.run_until_complete(runner.cleanup())
 
     async def start_site(
         self, app: web.Application, address: str = "0.0.0.0", port: int = 2080
